from requests import Session, RequestException
from typing import Literal, Dict, Any, Optional


class FailedCaptcha(Exception): pass
class InvalidKey(Exception): pass
class InvalidMethod(Exception): pass
class FailedBalance(Exception): pass


MethodTypes = Literal[
    "outlook", "twitter", "twitter_unlock", "roblox_signup", "roblox_login",
    "roblox_join", "ea", "github-signup", "demo", "roblox_wall", "airbnb-register"
]

BrowserTypes = Literal[
    "chrome", "opera", "edge", "firefox", "chrome mac", "firefox mac",
    "chrome linux", "firefox linux", "safari"
]

OperatingSystemTypes = Literal["windows", "mac", "linux"]


class CaptchaSolver:
    _session = Session()
    _base_url = "https://api.captchasolver.ai/"

    def __init__(self, method: MethodTypes, key: str):
        if not key or not isinstance(key, str):
            raise InvalidKey("CaptchaSolver key must be a non-null string")
        if method not in MethodTypes.__args__:
            raise InvalidMethod(f"{method} is not a valid method. Valid options: {MethodTypes.__args__}")

        self.key = key
        self.method = method

    def solve(
        self,
        proxy: str,
        blob: Optional[str] = None,
        method: Optional[MethodTypes] = None,
        browser: BrowserTypes = "firefox",
        operating_system: OperatingSystemTypes = "windows",
        version: int = 139,
    ) -> str:

        method = method or self.method

        if method not in MethodTypes.__args__:
            raise InvalidMethod(f"{method} is not a valid method. Valid options: {MethodTypes.__args__}")

        payload = {
            "key": self.key,
            "method": method,
            "proxy": proxy,
            "browser": browser,
            "os": operating_system,
            "version": version,
        }

        if blob:
            payload["blob"] = blob

        try:
            response = self._session.post(f"{self._base_url}api/solve", json=payload)
            response.raise_for_status()

            result: Dict[str, Any] = response.json()
            if "Failed:" in result.get("msg", ""):
                raise FailedCaptcha(result["msg"])

            token = result.get("token")
            if not token:
                raise FailedCaptcha("No token received from CaptchaSolver")

            return token

        except RequestException as e:
            raise FailedCaptcha(f"Network error while solving captcha: {e}") from e
        except (ValueError, KeyError) as e:
            raise FailedCaptcha(f"Invalid response format: {e}") from e

    def balance(self) -> int:
        try:
            response = self._session.get(
                f"{self._base_url}api/getBalance", params={"key": self.key}
            )
            response.raise_for_status()

            result: Dict[str, Any] = response.json()
            balance = result.get("balance")
            if balance is None:
                raise FailedCaptcha("No balance received from CaptchaSolver")

            return int(balance)

        except RequestException as e:
            raise FailedBalance(f"Network error while fetching balance: {e}") from e
        except (ValueError, KeyError) as e:
            raise FailedBalance(f"Invalid response format: {e}") from e



from capsolai import CaptchaSolver

cs = CaptchaSolver(
    "github-signup",
    key="C-AI-9646F42A4F98B2A4D385029284B8980A" # empty key for testing purposes
)

print(cs.balance())

print(cs.solve(
    proxy="http://user:pass@proxyserver:port",
    blob="blob_data",
    browser="firefox",
    operating_system="windows",
    version=139
))


__version__ = "0.1.0"

from .solver import CaptchaSolver, FailedCaptcha, InvalidKey, InvalidMethod, FailedBalance

__all__ = [
    "CaptchaSolver",
    "FailedCaptcha",
    "InvalidKey",
    "InvalidMethod",
    "FailedBalance",
]



from setuptools import setup, find_packages

setup(
    name="capsolai", 
    version="0.2.0",
    author="trailios",
    author_email="admin@bombing.lol",
    description="Python API wrapper for the Captcha Solver AI service",
    url="https://github.com/trailios/csai",
    license="Apache License 2.0",
    packages=find_packages(include=["capsolai", "capsolai.*"]),
    install_requires=[
        "requests>=2.25",
        "typing-extensions>=4.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.7",
)
